/*
* This file is part of QTGZManager, an open-source GUI for Slackware pkgtools.
* Copyright (C) 2006  Alexandre Albuquerque Arnt
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact e-mail : Alexandre Albuquerque Arnt <aaarnt@gmail.com>
* Program URL   : http://jtgzmanager.sf.net
*
*/

#include <iostream>
#include "package.h"

#include <QtCore>
#include <QMessageBox>
#include <QStandardItemModel>
#include <QStandardItem>

InstalledPkgListSingleton *InstalledPkgListSingleton::m_pinstance = 0;
FrozenPkgListSingleton *FrozenPkgListSingleton::m_pinstance = 0;

bool Package::isValid( const QString& pkgName ){
	QFileInfo fi(pkgName);
	if (fi.size() < 100) return false;
	else return true;
}

QString Package::getBaseName( const QString& p ){
	QString packageBaseName="";
	QString aux(p);
	int numberOfSegments = p.count('-')+1;
	int nameSegment = numberOfSegments-3;
	int a;

	for (int i=1; i<= nameSegment; i++){
		a=aux.indexOf("-");
		packageBaseName += aux.left(a);
		if (i<nameSegment) packageBaseName += "-";
		aux = aux.mid(a+1);
	}

	if (packageBaseName == "") packageBaseName += p.left(p.indexOf("-"));
	return packageBaseName;
}

QString Package::dumpInstalledPackageList(){
	QDateTime now = QDateTime::currentDateTime();
  QString dumpFileName = ctn_DUMP_FILE + now.toString ("dd_MM_yyyy_hh_mm_ss") + ".txt";
 
  QFile file(QDir::homePath() + QDir::separator() + dumpFileName);
  if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) return "";
  QTextStream out(&file);

  foreach (QString fileName, InstalledPkgListSingleton::instance()->getFileList()){
    QFileInfo fi( ctn_PACKAGES_DIR + QDir::separator() + fileName );
		QDateTime lastModifiedDate = fi.lastModified();
		out << fileName << " " << lastModifiedDate.toString("dd/MM/yyyy hh:mm:ss") << "\n";		
	}

	file.flush();
	file.close();

	return QDir::homePath() + QDir::separator() + dumpFileName;
}

//Here, we do the URL html link tag addition
QString Package::makeURLClickable( const QString &s ){
	QString sb = s;

	QRegExp rx("((ht|f)tp(s?))://(\\S)+[^\"|)|(|.|\\s|\\n]");
	QRegExp rx1("^|[\\s]+(www\\.)(\\S)+[^\"|)|(|.|\\s|\\n]"); 
	rx.setCaseSensitivity( Qt::CaseInsensitive );
	rx1.setCaseSensitivity( Qt::CaseInsensitive );

	int search = 0;
	int ini = 0;

	//First we search for the 1st pattern: rx
	while ( (ini = rx.indexIn( sb, search )) != -1 ){
		QString s1 = rx.cap();
		QString ns = "<a href=\"" + s1 + "\">" + s1 + "</a>";
		sb.replace( ini, s1.length(), ns);
		search = ini + (2*s1.length()) + 15;	
	}

	search = 0;
	ini = 0;
	//Now, we search for the 2nd pattern: rx1
	while ( (ini = rx1.indexIn( sb, search )) != -1 ){
		QString s1 = rx1.cap();
		QString ns;
		if (s1[0] == '\n') ns += "\n";

		int blanks = s1.count(	QRegExp("^|[\\s]+") );
		for (int i=0; i<blanks; i++) ns += " ";

		ns += "<a href=\"http://" + s1.trimmed() + "\">" + s1.trimmed() + "</a>";
		sb.replace( ini, s1.length(), ns);
		search = ini + (2*s1.length()) + 15;	
	}

	sb.replace(QRegExp("\n"), "<br>");
	return sb;
}

// Regular expression for "http://" -> ^((ht|f)tp(s?))\://([0-9a-zA-Z\-]+\.)+[a-zA-Z]{2,6}(\:[0-9]+)?(/\S*)?$
QString Package::getInformation( QString pkgName, bool installed ) {
	QFileInfo fi( pkgName );    
	if ( installed == false ) {
    //"tar -Oxzf lives-0.9.8-i486-1gas.tgz install/slack-desc > slack-desc" 
    QProcess tar;
    QStringList args;
    if (pkgName.endsWith(ctn_TGZ_PACKAGE_EXTENSION)) args << "-Oxzf";
    else args << "-OxJf";
    args << pkgName;
    args << "install/slack-desc";        
    tar.start("tar", args);

		QByteArray result("");	
		if (!tar.waitForFinished(100)){
			result = tar.readAllStandardOutput();					
			if (result == ""){
				tar.waitForFinished();		
				result = tar.readAllStandardOutput();					
			}
		}
    if (result == "") result = tar.readAllStandardOutput();
		QString pname = Package::getBaseName( fi.fileName() );
    QString sb = result;
    QString t = pname + ":";
    //t = t.replace("\n", "");
    //t.concat(":");
    
    int x = sb.indexOf(t, 0);
   	if (x > 0) sb.remove(0, x);
   	int begin=0;
   	x = sb.indexOf(t);
    while(x >= 0){        
   		if (x==0) sb.remove(x, x+t.length()+1);
   		else {
   			if (!sb.mid(x-1, x).contains("\\S")) sb.remove(x, (x+t.length()+1)-x);
   			else begin += pname.size();
   		}
    	x = sb.indexOf(t, begin);
    }	

		if ( !sb.isEmpty() ) return makeURLClickable(sb);
		else return 0;
	}
	else if ( installed == true ){
    QProcess tar;
    QStringList args;
  
    args << pkgName;
    tar.start("cat", args);        
    if (!(tar.waitForStarted())) return "";
    if (!(tar.waitForFinished())) return "";
    QByteArray result = tar.readAll();
    
    QString pname = Package::getBaseName( fi.fileName() );
		QString sb = result;  
    QString pkgSize;

		int y = sb.indexOf("UNCOMPRESSED PACKAGE SIZE", 0);
		if (y > 0) {
			y += 26;			
			while ( sb[y] != '\n' ){
				if ( sb[y].isDigit() ) pkgSize += sb[y];
				y++;
			}
		}
		if (!pkgSize.isEmpty()){
			bool ok;
			double d = pkgSize.toDouble( &ok );
			if ( ok == true ){
				if ( d > 1024 ){
					d /= 1024;
					pkgSize = QString::number( d, '.', 2 ) + " MB";
				}
				else pkgSize += " KB";
			} 				
		}

    QString t = pname + ":";
    //t = t.replace("\n", ""); 

    int x = sb.indexOf(t, 0);
  	if (x > 0) sb.remove(0, x);
      
    x = sb.indexOf("FILE LIST:", 0);
  	if (x > 0){
  		sb.remove(x, (sb.size()-1)-x);
   	}                      
   	
   	int begn=0;
   	x = sb.indexOf(t);
       
   	if (x == -1){
			if (pkgSize.isEmpty()) return "";
			else return QObject::tr("Installed size: %1").arg(pkgSize);
   	}
   		
   	while(x >= 0){        
   		if (x==0) sb.remove(x, (x+t.size()+1)-x);
   		else {
   			if (!sb.mid(x-1, x).contains("\\S")) sb.remove(x, (x+t.length()+1)-x);
   			else begn += pname.length();
   		}
     	x = sb.indexOf(t, begn);
    }	

		int z = sb.lastIndexOf(QRegExp("[\\n]{2,}"));
		if (z >= 0) sb.remove(z, sb.size()-z);
		
		//sb += "\n";
		if (!pkgSize.isEmpty()) sb += "\n\n" + QObject::tr("Installed size: %1").arg(pkgSize);
		
		if ( !sb.isEmpty() ) return makeURLClickable(sb);
		else return 0;
	}
	
	return 0;
}

QString Package::showRegExp( const QString& a, const QString& re ){
	QRegExp rex(re);	
  QString res = rex.indexIn(a) > -1 ? rex.cap(0) : ctn_NO_MATCH;
	return res;
}

Result Package::getStatus( const QString& arq ){       
  bool newPackageExtension = false;
  QString arqBaseName = Package::getBaseName(arq);

	if ( FrozenPkgListSingleton::instance()->indexOf( QRegExp(arqBaseName), 0 ) != -1){ 
		//QString pattern =getBaseName(arq);
    QString installedPackage = InstalledPkgListSingleton::instance()->getFileList().
                               filter( QRegExp(arqBaseName) )[0];
    int i = InstalledPkgListSingleton::instance()->getFileList().indexOf( QRegExp( installedPackage ) );
    return(Result(ectn_FROZEN, InstalledPkgListSingleton::instance()->getFileList().value(i)));
	}

  if (arq.endsWith(ctn_RPM_PACKAGE_EXTENSION)){
      return(Result(ectn_RPM, ""));
  }
  else{
    if (arq.endsWith(ctn_TXZ_PACKAGE_EXTENSION)) newPackageExtension = true;
    int i = InstalledPkgListSingleton::instance()->getFileList().indexOf(arq.left(arq.length()-4));
    if (i != -1){
      return(Result(ectn_INSTALLED, InstalledPkgListSingleton::instance()->getFileList().value(i)));
    }
  }

  Result res(ectn_NOT_INSTALLED, "");
  bool goAhead=false;

  QString res1 = showRegExp(arq, ctn_ER);
  if (res1 == ctn_NO_MATCH) {
    return Result(ectn_INTERNAL_ERROR, "");
  }

	QRegExp raux;
  QStringList lPackages2 = InstalledPkgListSingleton::instance()->getFileList().
                           filter( QRegExp(raux.escape( arqBaseName ) ));

  foreach ( QString pacote, lPackages2 ){      
    if ((arq == ".") || (arq == "..")) continue;        
               
    QString res2 = showRegExp(pacote, ctn_ER3);

    int a = res1.indexOf(res2);        
    if (a == 0){
      //It must first make the "brake string" test... 
      QStringList c = arq.split("-");
      QStringList d = pacote.split("-");

      if ((c.size() != 0) && (d.size() != 0) && (c.size() == d.size())){
        goAhead=true;
        for (int k = 0; k < c.size()-3; k++) {
	        if (c[k] != (d[k])) goAhead = false;  
        }

        if (goAhead){
          QString st1(c[c.size()-3]);
          QString st2(d[c.size()-3]);
          
          QStringList ver1 = st1.split(QRegExp("[._]"));
          QStringList ver2 = st2.split(QRegExp("[._]"));
                  
          bool bVer1Alpha=false, bVer2Alpha=false;                  
                  
          foreach (QString x, ver1){
            if (x.indexOf(QRegExp("[a-zA-Z]")) != -1){
              bVer1Alpha = true; 
            }
          }
          
          foreach (QString x, ver2){
            if (x.indexOf(QRegExp("[a-zA-Z]")) != -1){
              bVer2Alpha = true; 
            }
          }
 
					if (bVer1Alpha != bVer2Alpha) 
            return Result (ectn_OTHER_VERSION, pacote);

					else if (bVer1Alpha && bVer2Alpha){
     			  QString s1 ="", s2="";
     			  foreach (QString aux, ver1) s1 = s1 + aux;
     			  foreach (QString aux, ver2) s2 = s2 + aux;
    			  if (s1.compare(s2, Qt::CaseInsensitive) == 0) goAhead = true;
            else return Result (ectn_OTHER_VERSION, pacote);
					}	

					else if (!bVer1Alpha && !bVer2Alpha){ 
						if ((ver1.size()) == 1 || (ver2.size() == 1)){
	     			  QString s1="", s2="";
							foreach (QString aux, ver1) s1 = s1 + aux;
  	   			  foreach (QString aux, ver2) s2 = s2 + aux;	        			  
              if ((s1.indexOf(QRegExp(ctn_DATE_RELEASE)) != -1) ||
                  (s2.indexOf(QRegExp(ctn_DATE_RELEASE)) != -1)) return Result (ectn_OTHER_VERSION, pacote);
						}

						if (ver1.size() != ver2.size()) {
							if (ver1.size() > ver2.size()){
								int sub = ver1.size()-ver2.size();
								for(int a=0; a<sub; a++) ver2 << "0";
							}
							else {
								int sub = ver2.size()-ver1.size();
								for(int a=0; a<sub; a++) ver1 << "0";	
							}
						}

						for(int a=0; a<ver1.size(); a++){						
							bool equal = ver1[a].toInt() == ver2[a].toInt();						
							if ( equal == true ) continue;

							bool comp = ver1[a].toInt() > ver2[a].toInt();
							if ( comp == true )
                return Result ( ectn_SUPERIOR_VERSION, pacote );
							else if ( comp == false )
                return Result ( ectn_INFERIOR_VERSION, pacote );
						}
					}	

          if (goAhead){
            int n1=0, n2=0;	        			  
    			  QString s1="", s2="";
     			  foreach (QString aux, ver1) s1 = s1 + aux;
     			  foreach (QString aux, ver2) s2 = s2 + aux;	        			  
     			  int comp = s1.compare(s2, Qt::CaseInsensitive);
     			  bool equal = (comp == 0);
     			  
    				//We must take care with diverse releases of the same package (the final number).
     			  if (equal) {
							//We must make the platform test 
	     			  comp = c[c.size()-2].compare(d[c.size()-2], Qt::CaseInsensitive);
  	   			  equal = (comp == 0);							

							if (equal){
								//Here comes the release test!
                if (newPackageExtension)
                  c[c.size()-1] = c[c.size()-1].replace(ctn_TXZ_PACKAGE_EXTENSION, "");
                else c[c.size()-1] = c[c.size()-1].replace(ctn_TGZ_PACKAGE_EXTENSION, "");
								QString sn1, sn2;
								sn1 = showRegExp(c[c.size()-1], "[0-9]*");
								sn2 = showRegExp(d[c.size()-1], "[0-9]*");
	
   					  	bool ok1, ok2;
              	n1 = sn1.toInt(&ok1);
              	n2 = sn2.toInt(&ok2);
              	if (!(ok1) || !(ok2)) break; 
               
   					  	if (n1 > n2){	        				  
                  return Result(ectn_SUPERIOR_VERSION, pacote);
     				  	}		        				  
     				  	else if (n1 < n2){
                  return Result(ectn_INFERIOR_VERSION, pacote);
     					  }	
								else if (n1 == n2){
                  return Result(ectn_INSTALLED, pacote);
								}
     			  	}
							else{
                if (comp > 0) return Result(ectn_SUPERIOR_VERSION, pacote);
                else return Result(ectn_INFERIOR_VERSION, pacote);
							}
						}
            else if (!(equal)) {
 				  	  if (s1.size() != s2.size()){
 				  		  if (s1.size() < s2.size()){
 				  			  for(int j=0; j<s2.size()-s1.size(); j++){
 				  				  s1 = s1 + "0";
 				  			  }
 				  		  }
 				  		  else{
 				  			  for(int j=0; j<s1.size()-s2.size(); j++){
 				  				  s2 = s2 + "0";
 				  			  }
 				  		  }	        				  		  
 				  		  
 	        		  comp = s1.compare(s2, Qt::CaseInsensitive);
 				  	  }
 				  	  
 				  	  if (comp > 0){
                return Result(ectn_SUPERIOR_VERSION, pacote);
 					    }		        				  
 					  	
 					    else if (comp < 0){
                return Result(ectn_INFERIOR_VERSION, pacote);
	            }	        				  
	          }	  
	        }	  
	      }
	    }
    }
	}        
  
  return res; 						
}

QStringList Package::getContents( const QString& pkgName, bool installed ){
  FILE *file;
  char linebuf[1024];
  QString st, fn; 
	QStringList sl;	

	if ( installed ){
		fn = pkgName;
		file = fopen ( QFile::encodeName ( fn ),"r" );
		if ( file ){
			while ( fgets ( linebuf,sizeof ( linebuf ),file ) ){
        if ( !strcmp ( linebuf, ctn_FILELIST ) ){
					break;
				}
			}
			while ( fgets ( linebuf,sizeof ( linebuf ),file ) ){
				st = "/";
				st += linebuf;
				st.truncate ( st.length() -1 );
				if ( st.left ( 8 ) != "/install" && st.left ( 3 ) != "/./" ){
					sl << st;
				}
			}
			fclose ( file );
		}
	}
	else{
		QProcess tar;
	
		QStringList args;
    args << "-Otf";
		args << pkgName;
		QStringList rsl;

		tar.start ( "tar", args );
	
		if ( ! ( tar.waitForStarted() ) ) return QStringList();
		if ( ! ( tar.waitForFinished() ) ) return QStringList();
		QByteArray result = tar.readAllStandardError(); 
	
		QString aux(result);
		rsl = aux.split("\n", QString::SkipEmptyParts);
		
		if ( !rsl.isEmpty() ){
			rsl.removeFirst();
			rsl.replaceInStrings(QRegExp("^"), "/");
			rsl.sort();
		}
		
		//"/gzip: stdin: unexpected end of file"
    if (!rsl.contains("/gzip: stdin: unexpected end of file") &&
        !rsl.contains("/tar: Child returned status 1")) return rsl;
		else{ 
			rsl.clear();
			return rsl;
		}
	}
	
	sl.sort();
	return sl;
}	

QMap<QString, QStringList> Package::findFile( const QString& name ){
  FILE *file;
  char linebuf[1024];
  QString st;
  QString fn, dr = ctn_INSTALLED_PACKAGES_DIR;
	QMap<QString, QStringList> m;
    
  QDir d(ctn_INSTALLED_PACKAGES_DIR);
	if ( d.exists() ){

      QStringList slAux;
      const QFileInfoList list = d.entryInfoList();
      foreach ( QFileInfo fi, list )      {
        QCoreApplication::processEvents();

        if ( !fi.isDir() && fi.isReadable() ){
          fn = dr + fi.fileName();
          file = fopen ( QFile::encodeName ( fn ),"r" );
          if ( file ){
            QStringList sl;
            while ( fgets ( linebuf,sizeof ( linebuf ),file ) ){
              if ( !strcmp ( linebuf, ctn_FILELIST ) ){
                break;
              }
            }
            while ( fgets ( linebuf,sizeof ( linebuf ),file ) ){
              if ( QString::fromLocal8Bit ( linebuf ).indexOf ( name, 0, Qt::CaseInsensitive ) != -1 ){
                st = "/";
                st += linebuf;
                st.truncate ( st.length() -1 );
                if ( st.left ( 8 ) != "/install" ){
                  if (m.contains(fi.fileName())){
                    slAux = m.value(fi.fileName());
                    slAux.append(st);
                    m.insert(fi.fileName(), slAux);
                  }
                  else
                    m.insert(fi.fileName(), sl << st);

                  sl.clear();
                }
              }
            }
            fclose ( file );
          }
        }
      }
    }

  return m;
}

QString Package::showFullPathOfItem( const QModelIndex &index ){
  QString str;
  if (!index.isValid()) return str;

  const QStandardItemModel *sim = qobject_cast<const QStandardItemModel*>( index.model() );
  QStringList sl;
  QModelIndex nindex;
  sl << sim->itemFromIndex( index )->text();

  nindex = index;
  while (1){
    nindex = sim->parent( nindex );
    if ( nindex != sim->invisibleRootItem()->index() ) sl << sim->itemFromIndex( nindex )->text();
    else break;
  }
  str = QDir::separator() + str;

  for ( int i=sl.count()-1; i>=0; i-- ){
    if ( i < sl.count()-1 ) str += QDir::separator();
    str += sl[i];
  }

  return str;
}

QMap<QString, QStringList> Package::findFile( const QString& name, const QStandardItemModel *sim){
  QMap<QString, QStringList> m;
  QList<QStandardItem *> foundItems;
  QString fileName;
  QStringList listAux;

  if (sim->rowCount() == 0) return m;

  //QMessageBox::information(0, "Debug",
  //                         QString::number(number));

  foundItems = sim->findItems(name, Qt::MatchContains|Qt::MatchRecursive, 0);
  QString fullPath;

  foreach(QStandardItem *item, foundItems){
    QCoreApplication::processEvents();
    if (item->accessibleDescription().contains("directory ")) continue;

    if (item->parent()){
      fileName = item->text();
      fullPath = showFullPathOfItem(item->parent()->index());
    }
    if (item->parent() && item->parent()->hasChildren() &&
        item->parent() != item->model()->invisibleRootItem() &&
        m.contains(fullPath)){

      listAux = m.value(fullPath);
      listAux << fileName;
      m.insert(fullPath, listAux);
    }
    else if (item->parent() && item->parent() != item->model()->invisibleRootItem())
      m.insert(fullPath, listAux << fileName);

    listAux.clear();
  }

  return m;
}

QMap<QString, QStringList> Package::findPackage( const QString& name, const QString& searchDir ){
  QMap<QString, QStringList> m;

  QStringList nameFilters;
  nameFilters << "*" + ctn_TGZ_PACKAGE_EXTENSION << "*" + ctn_TXZ_PACKAGE_EXTENSION << "*" + ctn_RPM_PACKAGE_EXTENSION;

  QDirIterator *targetDir = new QDirIterator(searchDir, nameFilters,
                                             QDir::AllEntries,
                                             QDirIterator::Subdirectories);
  QString res;
  QString dir;
  QStringList fileNames, slAux;
  while (targetDir->hasNext()){
    QCoreApplication::processEvents();
    targetDir->next();
    res = targetDir->fileName();
    QFileInfo fi = targetDir->fileInfo();
    dir = fi.absolutePath();

    if (res.indexOf(name, 0, Qt::CaseInsensitive) != -1){
      fileNames << res;
      if (m.contains(dir)){
        slAux = m.value(dir);
        slAux.append(res);
        m.insert(dir, slAux);
      }
      else m.insert(dir, fileNames);

      fileNames.clear();
    }
  }

  delete targetDir;
  return m;
}

void Package::testSearchInDir(){

  QMap<QString, QStringList> map = findPackage("qt-", "/home/arnt/Packages");

  foreach (QString k, map.keys()){
    foreach( QStringList sl, map.values(k) ){
      for ( int c=0; c<sl.count(); c++ ){
        QMessageBox::information(0, "Found!", k + "/" + sl[c]);
      }
    }
  }
}

InstalledPkgListSingleton* InstalledPkgListSingleton::instance(){
  if (m_pinstance == 0) m_pinstance = new InstalledPkgListSingleton();
  return m_pinstance;
}

QStringList InstalledPkgListSingleton::getFileList(){
  return m_pkgList;
}

void InstalledPkgListSingleton::setFileSystemWatcher(QFileSystemWatcher* fsw){
  connect (fsw, SIGNAL(directoryChanged(const QString)),
                    this, SLOT(installedPkgDirChanged()));
}

InstalledPkgListSingleton::InstalledPkgListSingleton():QObject(){
  QDir scanPackages = QDir(ctn_PACKAGES_DIR);
  scanPackages.setFilter(QDir::Files);
  m_pkgList = scanPackages.entryList();
}

void InstalledPkgListSingleton::installedPkgDirChanged(){
  m_pinstance = 0;
}
